@@ -8,12 +8,12 @@ Static resources for L{twisted.web}.
 
 from __future__ import division, absolute_import
 
-import os
-import warnings
-import itertools
-import time
 import errno
+import itertools
 import mimetypes
+import os
+import time
+import warnings
 
 from zope.interface import implementer
 
@@ -22,7 +22,8 @@ from twisted.web import resource
 from twisted.web import http
 from twisted.web.util import redirectTo
 
-from twisted.python.compat import networkString, intToBytes, nativeString, _PY3
+from twisted.python.compat import (_PY3, intToBytes, nativeString,
+                                   networkString)
 from twisted.python.compat import escape
 
 from twisted.python import components, filepath, log
@@ -30,7 +31,7 @@ from twisted.internet import abstract, interfaces
 from twisted.python.util import InsensitiveDict
 from twisted.python.runtime import platformType
 from twisted.python.url import URL
-from twisted.python.versions import Version
+from incremental import Version
 from twisted.python.deprecate import deprecated
 
 if _PY3:
@@ -132,7 +133,7 @@ def loadMimeTypes(mimetype_locations=None, init=mimetypes.init):
 
     @param mimetype_locations: Optional. List of paths to C{mime.types} style
         files that should be used.
-    @type mimetype_locations: iterable of paths or C{None}
+    @type mimetype_locations: iterable of paths or L{None}
     @param init: The init function to call. Defaults to the global C{init}
         function of the C{mimetypes} module. For internal use (testing) only.
     @type init: callable
@@ -186,6 +187,15 @@ class File(resource.Resource, filepath.FilePath):
 
     @cvar childNotFound: L{Resource} used to render 404 Not Found error pages.
     @cvar forbidden: L{Resource} used to render 403 Forbidden error pages.
+
+    @ivar contentTypes: a mapping of extensions to MIME types used to set the
+        default value for the Content-Type header.
+        It is initialized with the values returned by L{loadMimeTypes}.
+    @type contentTypes: C{dict}
+
+    @ivar contentEncodings: a mapping of extensions to encoding types used to
+        set default value for the Content-Encoding header.
+    @type contentEncodings: C{dict}
     """
 
     contentTypes = loadMimeTypes()
@@ -223,7 +233,7 @@ class File(resource.Resource, filepath.FilePath):
             with a L{File} pointing to C{"foo.bar"}.
 
         @param registry: The registry object being used to handle this
-            request.  If C{None}, one will be created.
+            request.  If L{None}, one will be created.
         @type registry: L{Registry}
 
         @param allowExt: Ignored parameter, only present for backwards
@@ -235,7 +245,7 @@ class File(resource.Resource, filepath.FilePath):
         if ignoredExts in (0, 1) or allowExt:
             warnings.warn("ignoredExts should receive a list, not a boolean")
             if ignoredExts or allowExt:
-                self.ignoredExts = [b'*']
+                self.ignoredExts = ['*']
             else:
                 self.ignoredExts = []
         else:
@@ -263,11 +273,33 @@ class File(resource.Resource, filepath.FilePath):
 
     def getChild(self, path, request):
         """
-        If this L{File}'s path refers to a directory, return a L{File}
+        If this L{File}"s path refers to a directory, return a L{File}
         referring to the file named C{path} in that directory.
 
-        If C{path} is the empty string, return a L{DirectoryLister} instead.
+        If C{path} is the empty string, return a L{DirectoryLister}
+        instead.
+
+        @param path: The current path segment.
+        @type path: L{bytes}
+
+        @param request: The incoming request.
+        @type request: An that provides L{twisted.web.iweb.IRequest}.
+
+        @return: A resource representing the requested file or
+            directory, or L{NoResource} if the path cannot be
+            accessed.
+        @rtype: An object that provides L{resource.IResource}.
         """
+        if isinstance(path, bytes):
+            try:
+                # Request calls urllib.unquote on each path segment,
+                # leaving us with raw bytes.
+                path = path.decode('utf-8')
+            except UnicodeDecodeError:
+                log.err(None,
+                        "Could not decode path segment as utf-8: %r" % (path,))
+                return self.childNotFound
+
         self.restat(reraise=False)
 
         if not self.isdir():
@@ -288,12 +320,13 @@ class File(resource.Resource, filepath.FilePath):
             if fpath is None:
                 return self.childNotFound
 
+        extension = fpath.splitext()[1]
         if platformType == "win32":
             # don't want .RPY to be different than .rpy, since that would allow
             # source disclosure.
-            processor = InsensitiveDict(self.processors).get(fpath.splitext()[1])
+            processor = InsensitiveDict(self.processors).get(extension)
         else:
-            processor = self.processors.get(fpath.splitext()[1])
+            processor = self.processors.get(extension)
         if processor:
             return resource.IResource(processor(fpath.path, self.registry))
         return self.createSimilarFile(fpath.path)
@@ -320,7 +353,7 @@ class File(resource.Resource, filepath.FilePath):
         @return: A list C{[(start, stop)]} of pairs of length at least one.
 
         @raise ValueError: if the header is syntactically invalid or if the
-            Bytes-Unit is anything other than 'bytes'.
+            Bytes-Unit is anything other than "bytes'.
         """
         try:
             kind, value = range.split(b'=', 1)
@@ -370,7 +403,7 @@ class File(resource.Resource, filepath.FilePath):
         being served (and so has the value of C{getFileSize()} as an indirect
         input).
 
-        Either but not both of start or end can be C{None}:
+        Either but not both of start or end can be L{None}:
 
          - Omitted start means that the end value is actually a start value
            relative to the end of the resource.
@@ -383,9 +416,9 @@ class File(resource.Resource, filepath.FilePath):
         If this range doesn't overlap with any of this resource, C{(0, 0)} is
         returned, which is not otherwise a value return value.
 
-        @param start: The start value from the header, or C{None} if one was
+        @param start: The start value from the header, or L{None} if one was
             not present.
-        @param end: The end value from the header, or C{None} if one was not
+        @param end: The end value from the header, or L{None} if one was not
             present.
         @return: C{(offset, size)} where offset is how far into this resource
             this resource the range begins and size is how long the range is,
@@ -433,7 +466,7 @@ class File(resource.Resource, filepath.FilePath):
         @param request: The Request object.
         @param startAndEnd: A 2-tuple of start of the byte range as specified by
             the header and the end of the byte range as specified by the header.
-            At most one of the start and end may be C{None}.
+            At most one of the start and end may be L{None}.
         @return: A 2-tuple of the offset and size of the range to return.
             offset == size == 0 indicates that the request is not satisfiable.
         """
@@ -476,7 +509,7 @@ class File(resource.Resource, filepath.FilePath):
         @param request: The Request object.
         @param byteRanges: A list of C{(start, end)} values as specified by
             the header.  For each range, at most one of C{start} and C{end}
-            may be C{None}.
+            may be L{None}.
         @return: See above.
         """
         matchingRangeFound = False
@@ -526,7 +559,7 @@ class File(resource.Resource, filepath.FilePath):
         This method is not appropriate for requests for multiple byte ranges;
         L{_doMultipleRangeRequest} will set these headers in that case.
 
-        @param request: The L{Request} object.
+        @param request: The L{twisted.web.http.Request} object.
         @param size: The size of the response.  If not specified, default to
             C{self.getFileSize()}.
         """
@@ -545,7 +578,7 @@ class File(resource.Resource, filepath.FilePath):
 
         This method will also set the response code and Content-* headers.
 
-        @param request: The L{Request} object.
+        @param request: The L{twisted.web.http.Request} object.
         @param fileForReading: The file object containing the resource.
         @return: A L{StaticProducer}.  Calling C{.start()} on this will begin
             producing the response.
@@ -684,9 +717,9 @@ class StaticProducer(object):
         """
         Stop producing data.
 
-        L{IPullProducer.stopProducing} is called when our consumer has died,
-        and subclasses also call this method when they are done producing
-        data.
+        L{twisted.internet.interfaces.IProducer.stopProducing}
+        is called when our consumer has died, and subclasses also call this
+        method when they are done producing data.
         """
         self.fileObject.close()
         self.request = None
@@ -870,16 +903,22 @@ class DirectoryLister(resource.Resource):
         B{type} and B{encoding}.
     @type linePattern: C{str}
 
+    @ivar contentTypes: a mapping of extensions to MIME types used to populate
+        the information of a member of this directory.
+        It is initialized with the value L{File.contentTypes}.
+    @type contentTypes: C{dict}
+
     @ivar contentEncodings: a mapping of extensions to encoding types.
+        It is initialized with the value L{File.contentEncodings}.
     @type contentEncodings: C{dict}
 
     @ivar defaultType: default type used when no mimetype is detected.
     @type defaultType: C{str}
 
     @ivar dirs: filtered content of C{path}, if the whole content should not be
-        displayed (default to C{None}, which means the actual content of
+        displayed (default to L{None}, which means the actual content of
         C{path} is printed).
-    @type dirs: C{NoneType} or C{list}
+    @type dirs: L{None} or C{list}
 
     @ivar path: directory which content should be listed.
     @type path: C{str}
